#!/usr/bin/env python
# -*- coding: utf-8 -*-

# List the vebus devices on the DBus

import dbus
import os
import sys
from dbus.mainloop.glib import DBusGMainLoop

# our own packages
sys.path.insert(1, os.path.join(os.path.dirname(__file__), 'ext/velib_python/'))
from vedbus import VeDbusItemExport, VeDbusItemImport

# DBus mainloop shoul already be running
# DBusGMainLoop(set_as_default=True)

VEBUS_PATH = "com.victronenergy.vebus"
MK2TUNNEL = "/Interfaces/Mk2/Tunnel"

def vebus_list():
	# Connect to the sessionbus. Note that on ccgx we use systembus instead.
	dbusConn = dbus.SessionBus() if 'DBUS_SESSION_BUS_ADDRESS' in os.environ else dbus.SystemBus()

	# dictionary containing the different items
	dbusObjects = {}

	vebusList = []

	for item in dbusConn.list_names():
		if item.find(VEBUS_PATH) != -1:
			vebusList.append(item)

	systems = []

	for item in vebusList:
		dbusObject = VeDbusItemImport(dbusConn, item, '/ProductName') # TODO implement /CustomName
		if dbusObject.get_value() != None:
			system = { "servicename" : str(item), "readablename" : str(dbusObject.get_value())}
			systems.append(system)

	return {"systems" : systems}


class VebusList:
	def get_list(self):
		return vebus_list()
	def service_to_interface(self, serviceName):
		systemDict = vebus_list()

		for item in systemDict["systems"]:
			if item["servicename"] == serviceName:
				return "dbus://" + serviceName + MK2TUNNEL

		raise ValueError

""" Test code
DBusGMainLoop(set_as_default=True)
test_vbl = VebusList()
print test_vbl.service_to_interface("com.victronenergy.vebus.ttyO1")
print test_vbl.service_to_interface("com.blabla.vebus.ttyO1")
"""
